package mvd.jester.simulator.internals.schmidmottok;

import java.util.HashSet;
import java.util.Optional;
import java.util.Set;
import mvd.jester.model.Segment;
import mvd.jester.simulator.internals.JobContextInterface;

/**
 * Segment
 */
public class SegmentContext {
    private final Segment segment;
    private final Set<JobContextInterface> jobs;
    private final Set<TaskletContext> tasklets;

    public SegmentContext(TaskContext taskContext, Segment segment) {
        this.segment = segment;
        jobs = new HashSet<>();
        tasklets = new HashSet<>();

        for (int j = 0; j < segment.getNumberOfJobs(); ++j) {
            jobs.add(new JobContext(taskContext, this));
        }

        for (int j = 0; j < segment.getNumberOfTasklets(); ++j) {
            tasklets.add(new TaskletContext(taskContext, this));
        }
    }


    /**
     * @return the segment
     */
    public Segment getSegment() {
        return segment;
    }

    Optional<JobContextInterface> getNextJob() {
        boolean taskletAvailable = tasklets.stream()
                .anyMatch(t -> !t.getCurrentJob().isPresent() && t.checkExecutionTime());

        if (taskletAvailable) {
            return jobs.stream()
                    .filter(j -> !j.getCurrentProcessor().isPresent() && j.checkExecutionTime())
                    .findFirst();
        } else {
            return Optional.empty();
        }
    }

    Optional<TaskletContext> getNextTasklet() {
        return tasklets.stream()
                .filter(t -> !t.getCurrentJob().isPresent() && t.checkExecutionTime()).findFirst();
    }



    @Override
    public String toString() {
        return "(nJobs=" + segment.getNumberOfJobs() + ", nTasklets="
                + segment.getNumberOfTasklets() + ", taskletWcet=" + segment.getTaskletWcet() + ")";
    }
}
